import argparse
import os
from time import sleep
import traceback

from .encoding_converter import EncodingConverter

exclude_files = ['.DS_Store']
include_extensions = ['txt']


def main(*args, **kwargs):
    ap = argparse.ArgumentParser(description='Guess and convert '
                                 'file encodings from source to '
                                 'destination folders')
    ap.add_argument('input_dir',
                    help='Input directory',
                    type=str)
    ap.add_argument('output_dir',
                    help='Output directory',
                    type=str)
    ap.add_argument('-ie', '--input_encoding',
                    help='Input enconding',
                    default='guess')
    ap.add_argument('-gl', '--guess_length',
                    help='Number of bytes to use for guessing. -1=all',
                    default=1024)
    ap.add_argument('-oe', '--output_encoding',
                    help='Output enconding',
                    default='utf-8')
    ap.add_argument('-ow', '--overwrite',
                    help='When set, won\'t confirm when attempting to '
                    'overwrite.',
                    action='store_true')
    ap.add_argument('-i', '--interval',
                    help='Loop interval in seconds.',
                    default=2)
    args = ap.parse_args()
    if not os.path.isdir(args.input_dir):
        print('Input directory is not a directory')
        exit(1)
    if not os.path.isdir(args.output_dir):
        print('Output directory is not a directory')
        exit(1)
    args_dict = vars(args)
    print(
        f'****************************************\n'
        f'*       Encoding Portal starting       *\n'
        f'****************************************\n'
        f'src: "{args.input_dir}" ({args.input_encoding})\n'
        f'                   |                    \n'
        f'                   V                    \n'
        f'dst: "{args.output_dir}" ({args.output_encoding})'
    )
    failure_mark = 'encodefailed.'
    try:
        print(f'Checking every {args.interval:.1f} seconds...')
        while True:
            sleep(args.interval)
            filenames = [f for f in os.listdir(args.input_dir)
                         if os.path.isfile(os.path.join(args.input_dir, f))]
            for filename in filenames:
                if filename.startswith(failure_mark) or filename in \
                        exclude_files:
                    continue
                if filename.split('.')[-1] not in include_extensions:
                    continue
                print(f'\nNew file: {filename}')
                ec_args = args_dict.copy()
                input_path = os.path.join(args.input_dir, filename)
                ec_args['input_file'] = input_path
                output_path = os.path.join(args.output_dir, filename)
                ec_args['output_file'] = output_path
                ec = EncodingConverter(**ec_args)
                if ec.input_encoding.lower() == 'guess':
                    guess_result = ec.guess()
                    print(f'Guess: {guess_result}')
                    ec.input_encoding = guess_result['encoding']
                try:
                    if ec.convert():
                        print(f'Wrote to {ec.output_file} in '
                              f'{ec.output_encoding}')
                        os.remove(input_path)
                    else:
                        raise Exception(f'Conversion failed.')
                except Exception as e:
                    print(e)
                    traceback.print_exc()
                    os.rename(input_path, os.path.join(
                        args.input_dir, f'{failure_mark}{filename}'))
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



import argparse

from .encoding_converter import EncodingConverter

def main(*args, **kwargs):
    ap = argparse.ArgumentParser(description='Guess and convert between encodings')
    ap.add_argument('input_file',
                    help='Input file path',
                    type=str)
    ap.add_argument('-ie', '--input_encoding',
                    help='Input enconding',
                    default='guess')
    ap.add_argument('-gl', '--guess_length',
                    help='Number of bytes to use for guessing. -1=all',
                    default=1024)
    ap.add_argument('-oe', '--output_encoding',
                    help='Output enconding',
                    default='utf-8')
    ap.add_argument('-of', '--output_file',
                    help='Output file path',
                    default='')
    ap.add_argument('-ow', '--overwrite', 
                    help='When set, won\'t confirm when attempting to overwrite.',
                    action='store_true')
    args = ap.parse_args()
    ec = EncodingConverter(**vars(args))
    if ec.input_encoding.lower() == 'guess':
        guess_result = ec.guess()
        print(f'Guess: {guess_result}')
        ec.input_encoding = guess_result['encoding']
    if ec.convert():
        print(f'Wrote to {ec.output_file} in {ec.output_encoding}')
    else:
        print(f'Conversion failed.')


if __name__  == '__main__':
    main()


import argparse
import os
from time import sleep

from .encoding_converter import EncodingConverter

exclude_files = ['.DS_Store']
include_extensions = ['txt']

def main(*args, **kwargs):
    ap = argparse.ArgumentParser(description='Guess text encoding of files in folder')
    ap.add_argument('input_dir',
                    help='Input directory',
                    type=str)
    ap.add_argument('-gl', '--guess_length',
                    help='Number of bytes to use for guessing. -1=all',
                    default=1024)
    args = ap.parse_args()
    if not os.path.isdir(args.input_dir):
        print('Input directory is not a directory')
        exit(1)
    args_dict = vars(args)
    filenames = [f for f in os.listdir(args.input_dir)
                if os.path.isfile(os.path.join(args.input_dir, f))]
    for filename in filenames:
        if filename in exclude_files or filename.split('.')[-1] not in include_extensions:
            continue
        ec_args = args_dict.copy()
        input_path = os.path.join(args.input_dir, filename)
        ec_args['input_file'] = input_path
        ec = EncodingConverter(**ec_args)
        if ec.input_encoding.lower() == 'guess':
            guess_result = ec.guess()
            print(f'Guess for {filename}:\n{guess_result}')
            ec.input_encoding = guess_result['encoding']

if __name__  == '__main__':
    main()


import chardet
import os
from shutil import copyfile

class EncodingConverter:
    def __init__(self,
                 *args,
                 input_file=None,
                 input_encoding='guess',
                 guess_length=1024,
                 output_encoding='utf-8',
                 output_file=None,
                 overwrite=False,
                 **kwargs):
        self.input_file = input_file
        self.input_encoding = input_encoding
        self.guess_length = guess_length
        self.output_encoding = output_encoding
        self.output_file = output_file
        self.overwrite = overwrite
        for k, v in kwargs.items():
            setattr(self, k, v)

    @property
    def output_file(self):
        file_path = self._output_file
        # Create output file path if not specified
        if file_path == '':
            file_path = f'{self.input_file}.{self.output_encoding}'
        # Create output file path if output is specified as directory
        if os.path.isdir(file_path):
            input_file_name = os.path.basename(self.input_file)
            file_path = os.path.join(file_path, input_file_name)
        return file_path

    @output_file.setter
    def output_file(self, value):
        self._output_file = value

    def guess(self):
        sample = self._read(length=self.guess_length)
        return chardet.detect(sample)

    def convert(self, silent=False):
        try:
            # Confirm overwrite
            if os.path.exists(self.output_file) and not self.overwrite:
                overwrite_ok = input('Output file exists. Overwrite?\n(y/n): ')
                if overwrite_ok.lower() != 'y':
                    if not silent:
                        print('Abort writting.')
                    return False
            if not silent:
                self._print_file_size()
            if self.guess()['encoding'].lower() == 'utf-8':
                # No need to convert; just copy
                copyfile(self.input_file, self.output_file)
                return True
            print('Reading...')
            filebytes = self._read()
            print('Decoding...')
            decoded_str = filebytes.decode(
                self.input_encoding, errors='ignore')
            print('Encoding and writing...')
            with open(self.output_file, 'w', encoding=self.output_encoding) \
                    as file:
                file.write(decoded_str)
            return True
        except Exception as e:
            raise Exception(
                f'Failed to write to file {self.output_file}. ') from e

    def _print_file_size(self):
        filesize = os.stat(self.input_file).st_size
        units = ['B', 'KB', 'MB', 'GB', 'TB']
        unit_idx = 0
        while filesize >= 1024:
            unit_idx += 1
            filesize /= 1024
        print(f'File size: {filesize:.1f} {units[unit_idx]}')

    def _read(self, length=-1):
        try:
            # Blocked reading is a lot faster than taking a potentially very
            # large file into RAM
            block_size = 8192
            byte_lines = []
            rough_len = 0
            with open(self.input_file, 'rb') as file:
                while True:
                    block = file.readlines(block_size)
                    if not block:
                        break
                    byte_lines.extend(block)
                    rough_len += block_size
                    if length > -1 and rough_len >= length:
                        break
                filebytes = b''.join(byte_lines)
                return filebytes[:length]
        except Exception as e:
            raise Exception(f'Failed to read file {self.input_file}. ') from e


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setuptools.setup(
    name="enconv",
    version="0.0.4",
    author="Joy Yeh",
    author_email="joyyeh.tw@gmail.com",
    description="Encoding conversion utilities based on chardet.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/joy13975/enconv",
    packages=setuptools.find_packages(),
    scripts=['bin/enconv', 'bin/enportal', 'bin/encheck'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=required,
)
